"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _helperPluginUtils = require("@babel/helper-plugin-utils");

var _pluginSyntaxOptionalChaining = _interopRequireDefault(require("@babel/plugin-syntax-optional-chaining"));

var _core = require("@babel/core");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _default = (0, _helperPluginUtils.declare)(function (api) {
  api.assertVersion(7);
  return {
    name: "babel-plugin-logical-optional-chaining",
    inherits: _pluginSyntaxOptionalChaining["default"],
    visitor: {
      OptionalMemberExpression: function OptionalMemberExpression(path) {
        // Flag to preserve Computed Property Name Format
        var isPropertyComputed = path.node.computed; // Logically Chain the Object with an Empty Object using OR operation

        var chainExpression = _core.types.logicalExpression("||", path.node.object, _core.types.objectExpression([])); // If the Object is an Array, then logically chain it with an empty array


        if (path.node.property.type === "NumericLiteral") {
          chainExpression = _core.types.logicalExpression("||", path.node.object, _core.types.arrayExpression([]));
        } // Replace the original Object with Logically ORed Chain Expression in a Parenthesis


        path.replaceWith(_core.types.memberExpression(_core.types.parenthesizedExpression(chainExpression), path.node.property, isPropertyComputed));
      },
      OptionalCallExpression: function OptionalCallExpression(path) {
        var functionIdentifierString = _core.types.stringLiteral('function');

        var functionCallExpression = _core.types.callExpression(path.node.callee, path.node.arguments);

        var typeCheck = _core.types.unaryExpression('typeof', path.node.callee);

        var isTypeAFunction = _core.types.binaryExpression('===', typeCheck, functionIdentifierString);

        var functionCheckExpression = _core.types.parenthesizedExpression(_core.types.logicalExpression('&&', path.node.callee, isTypeAFunction));

        path.replaceWith(_core.types.logicalExpression('&&', functionCheckExpression, functionCallExpression));
      }
    }
  };
});

exports["default"] = _default;